	org 28672

; ZX to PMD image convertor
; Input: 	SCREEN$ at 49152
; Output: 	PMD mem dump at 32768, len: 16384
; Main routine does bitmap conversion (28672)
; For attributes call ORG + 3 (28675)
 
main:	jp	bitmap		; Do bitmap conversion
	jp	colors		; Do color converion
	jp	pack		; Pack output to 12k + depack routine
	jp	rwrt		; Rewrite depack routine + ldir
	jp	flush		; Flush buffers

colors: ld 	a,0		; Do attribute only conversion

nrow:	ld	de,22528	; Start of ZX Scrn Attr file
	call 	zxattr		; DE = zx attr
	ld	hl,32768	; Start of PMD SCRN
	call 	pmdlin		; HL = PMD "Attr"

	push	af		; Store line counter

	ld 	b,43		; 43 PMD attributes for 32 ZX attrs
atrbyt:	push	bc

	ld	a,255
	ld	(pix),a		; Assume we draw something - default
	ld	a,0
	ld	(xpap),a	; XOR paper - not

	ld	a,(de)		; Get ZX attr
	and 	%00000111	; Mask only INK value

	cp 	0
	jr	nz,nblk
	ld	a,b		; Are we on last column?
	cp	1
	jr	nz,ordnc	; NO -> Ordinary column - not last

	ld	a,15		; Last column mask (only 4 pixels)
	ld	(xpap),a
	jr	cnt1		; Continue

ordnc:	ld	a,63
	ld	(xpap),a	; XOR paper - active
cnt1:	ld	a,(de)
	and	%00111000
	rrca
	rrca
	rrca	

	cp 0            ; We have color in A
	jr z,blk	; Black ink black paper - no pixels
nblk:	cp 1
	jr z,blu
	cp 2
	jr z,red
	cp 3
	jr z,mgt
	cp 4
	jr z,grn
	cp 5
	jr z,cya
	cp 6
	jr z,yel

; white
   	ld a,%11000000
   	ld (fl1),a
   	xor a
   	ld (fl2),a
   	jr drw

blk:	xor a
	ld (pix),a   	; We draw nothing:
	jr drw		; if paper 0 (default PMD) and ink=0 => no pixels

blu: 	ld a,%10000000
	ld (fl1),a
	ld (fl2),a
	jr drw

red:   	ld a,%01000000
	ld (fl1),a
	ld (fl2),a
	jr drw

grn:    xor a
   	ld (fl1),a
   	ld (fl2),a
   	jr drw

mgt:	ld a,%11000000
   	ld (fl1),a
   	ld (fl2),a
   	jr drw

cya:   	xor a
   	ld (fl1),a
   	ld a,%10000000
   	ld (fl2),a
   	jr drw

yel:   xor a
   	ld (fl1),a
   	ld a,%01000000
   	ld (fl2),a
	
drw:	push 	hl
	push 	de	

	ld	de,64

	ld	b,4

cpls:	ld	c,(hl)		; Get pixels from PMD
	ld	a,(xpap)
	xor	c
	ld	c,a
	ld	a,(pix)		; Get mask from PIX
	and	c		; Combine mask + pixels
	ld	c,a		; Store temporarily in C
	ld	a,(fl1)		; Get new colors attribute
	or	c		; Now pixels are combined with color
	ld 	(hl),a		; And stored back

	add	hl,de		; 1st microline

	ld	c,(hl)		; Get pixels from PMD
	ld	a,(xpap)
	xor	c
	ld	c,a
	ld	a,(pix)		; Get mask from PIX
	and	c		; Combine mask + pixels
	ld	c,a
	ld	a,(fl2)		; Get new colors attribute
	or	c		; Now pixels are combined with color
	ld 	(hl),a		; And stored back

	add	hl,de		; 2nd microline

	djnz	cpls		; Do attr 'couples' - 4 times
	
	pop	de
	pop	hl

	inc 	hl		; Increment PMD pointer
	pop	bc

	ld	a,b		; Check last
	cp	2
	jr	z,skpinc	; If last PMD byte reached, use previous ZX attr.

	ld	a,b		; Increment only when 1,2 & 4; skip 3
	and	3
	cp	2
	jr	z,skpinc

	inc	de

skpinc:	dec	b

	jp	nz,atrbyt	; Attr bytes

	pop	af

	inc	a
	cp	24		; Repeat for 24 lines
	jp 	nz,nrow

	ret		; End of attribute conversion

;*********************
;*  Attr subroutines *
;*********************

; Calculates beginning of the line of ZX attributes
; Input:	A  - line number
;		DE - start offset of ZX Screen
; Out:		DE - address of source attribute byte
zxattr:	or	a
	ret	z

	push	af
	push	hl
	ex	de,hl

	ld	de,32	
attrnl:	add	hl,de
	dec	a
	jr	nz,attrnl	; Next attr line		

	ex 	de,hl
	pop	hl	
	pop	af

	ret		

; Main bitmap conversion routine
bitmap:	call	getzxs	; Get ZX screen 1st
	call	clspmd	; Delete converted screen

	ld a,0

nextl:	ld de,16384
	call linadr
	ex de,hl	; ZX address now in DE	

	ld hl,32768	; Beginning of PMD screen
	call pmdlin	; Now PMD address in HL
	
	push af		; Store line counter				

	ld b,8		; uLines within one line
line1:	push bc
	push de
	push hl

	ld b,11		; # of tripletes
bytes:	ld a,b
	ld (tripl),a
	push bc

	ld c,0	
	ld a,(de)	; Get ZX byte (1st)
	ld b,6
byteA:	rlca
	rr c
	djnz byteA

	scf		; Wrap byte for PMD screen
	ccf
	rr c
	rr c

	ld (hl),c	; Store  PMD byte (1st)
	inc hl

	ld c,0
	rlca
	rr c
	rlca
	rr c

	inc de		; Next ZX byte (2nd)
	ld a,(de)
	ld b,4
byteB1:	rlca
	rr c
	djnz byteB1		

	scf
	ccf
	rr c
	rr c

	ld (hl),c	; Store PMD byte (2nd)
	inc hl
	
	ld c,0
	ld b,4
byteB2: rlca
	rr c	
	djnz byteB2

	inc de

	ld   a,(tripl)
	cp   1
	ld   a,0
	jr   z,zxlst	; Not last byte od zx

	ld a,(de)	; Get ZX byte (3rd)

zxlst:	rlca
	rr c
	rlca
	rr c
	scf
	ccf
	rr c
	rr c

	ld (hl),c	; Store PMD byte (3rd)
	inc hl

	ld   b,a	
	ld   a,(tripl)
	cp   1
	jr   z,skplst	; Need to skip last byte of ZX
	ld   a,b

	ld b,6	
byteB3:	rlca
	rr c
	djnz byteB3
	scf
	ccf
	rr c
	rr c
	ld (hl),c	; Store PMD byte (4rd)
; 3-4 conversion done	 	

	inc hl
	inc de

skplst:	pop bc
	djnz bytes	

	pop hl
	ld de,64
	add hl,de

	pop de
	inc d
	
	pop bc
	djnz line1

	pop af
	inc a
	cp 24
	jp nz,nextl

	ret

; Compute PMD line start
; Compute the line address
; Input:        A - line address (0-23)
;		HL - address of PMD screen
; Output:       HL - real line begining address
pmdlin: or	a
	ret	z	; Nothing to do for line 0
	push 	af
	push 	de
	
	ld   	de,512
pmd1:	add 	hl,de
	dec 	a
	jr 	nz,pmd1
	
	pop	de
	pop	af
	ret

	
; Compute the line address
; Input:        A - line address (0-23)
;               DE - offset in memory (default 16384) (16384-58623)
; Output:       HL - real line begining address
linadr: push 	af
	push    af

        and     $07     ; lower byte will be calculated
        ld      b,a
        ld      c,0
        call    oe91

        pop     af

        and     $18     ; Get only 5th and 4th bit
        or      d
        ld      h,a
        ld      l,c
	pop 	af
	ret

; ROM routines - BC = 32*CB (0E91)
oe91:   ld   h,c
	ld   l,b
	add   hl,hl
	add   hl,hl
	add   hl,hl
	add   hl,hl
	add   hl,hl
	ld   b,h
	ld   c,l
	ret	

; Clear output
clspmd:	ld hl,32768
	ld de,32769
	ld bc,16383
	ld (hl),0
	ldir
	ret

getzxs:	ld hl,49152
	ld de,16384
	ld bc,6912
	ldir
	ret

; Pack routine
pack:	ld hl,32768+48		; Skip 1st microline - already OK
	ld d,h
	ld e,l

	ld b,255

uline:  push bc

	ld bc,48
	push de
	ld de,16
	add hl,de
	pop de
	ldir

	pop bc
	djnz uline	; Pack microline
	
	ret

;	org 32768
; Depack (8080  instructions only)
depack: ld hl,edpk-depack	; Source address
	ld de,49152	; Destination address

	ld c,255	; Microlines
field:	ld b,48		; Columns

dmicro:	ld a,(hl)
	ld (de),a
	
	inc hl
	inc de
	dec b
rw1:	jp nz,dmicro	

	ld b,16
skp16:	inc de
	dec b
rw2:	jp nz,skp16

	dec c
rw3:	jp nz,field
	
	ret		; End depack
edpk:	nop

; Rewerite absolute jumps for depack
rwrt:	ld a,dmicro-depack
	ld (rw1+1),a

	ld a,skp16-depack
	ld (rw2+1),a

	ld a,field-depack
	ld (rw3+1),a

	xor a
	ld (rw1+2),a
	ld (rw2+2),a
	ld (rw3+2),a

	ld hl,depack
	ld de,32768-(edpk-depack)
	ld bc,edpk-depack
	ldir

	ret

flush:	ld hl,32000
	ld de,32001
	ld (hl),0
	ld bc,33535
	ldir
	ret

tripl:  db 0
fl1:	db 0
fl2:	db 0
pix:	db 255
xpap:	db 0
